import 'package:flutter/material.dart';

import 'package:flutter/services.dart';
import 'package:dvote/dvote.dart';

final messageToEncrypt = 'This is the super secret text message';
final passphrase = "One key to rule them all";

class EncryptionScreen extends StatefulWidget {
  @override
  _EncryptionScreenState createState() => _EncryptionScreenState();
}

class _EncryptionScreenState extends State<EncryptionScreen> {
  String _cipherText = "-";
  String _decryptedMessage = "-";
  String _error;

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String cipherText, decryptedText;
    String error;

    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      cipherText = await encryptString(messageToEncrypt, passphrase);
      decryptedText = await decryptString(cipherText, passphrase);
    } on PlatformException catch (err) {
      error = err.message;
    } catch (err) {
      error = err;
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    if (error != null) {
      setState(() {
        _error = error;
      });
      return;
    }

    setState(() {
      _cipherText = cipherText;
      _decryptedMessage = decryptedText;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (_error != null) {
      return Scaffold(
        appBar: AppBar(
          title: const Text('AES encryption'),
        ),
        body: Container(
          child: Text("Error: " + _error),
        ),
      );
    }

    final encryptionData = '''Original message:
$messageToEncrypt\n
Encryption key:
$passphrase\n
Encrypted message:
$_cipherText\n
Decrypted message:
$_decryptedMessage''';

    return Scaffold(
      appBar: AppBar(
        title: const Text('AES encryption'),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.all(16),
            child: Column(
              children: <Widget>[
                Text("256-bit AES-GCM"),
                SizedBox(height: 30),
                Text(encryptionData),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
