import 'dart:io';
import 'dart:convert';
import 'package:flutter/material.dart';

import 'package:dvote/dvote.dart';

final networkId = "goerli";
final entityId =
    "0x180dd5765d9f7ecef810b565a2e5bd14a3ccd536c442b3de74867df552855e85";

class MetadataScreen extends StatefulWidget {
  @override
  _MetadataScreenState createState() => _MetadataScreenState();
}

class _MetadataScreenState extends State<MetadataScreen> {
  EntityMetadata _entityMeta;
  ProcessMetadata _processMeta;
  bool _loading = true;
  Exception _error;

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    EntityMetadata entityMeta;
    ProcessMetadata processMeta;
    Exception error;

    EntityReference entity = EntityReference();
    entity.entityId = entityId;
    entity.entryPoints.addAll(["https://rpc.slock.it/goerli"]);
    GatewayInfo gwInfo;
    DVoteGateway dvoteGw;
    Web3Gateway web3Gw;

    try {
      gwInfo = await getRandomDefaultGatewayDetails(networkId);
      dvoteGw = DVoteGateway(gwInfo.dvote, publicKey: gwInfo.publicKey);
      web3Gw = Web3Gateway(gwInfo.web3);

      entityMeta = await fetchEntity(entity, dvoteGw, web3Gw);
    } on SocketException {
      error = Exception("Connecting to the Gateway failed");
    } on Exception catch (err) {
      if (err is String) {
        error = err;
        print(err);
      } else {
        error = Exception("Unable to retrieve the entity metadata");
      }
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    if (error != null) {
      setState(() {
        _error = error;
        _loading = false;
      });
      return;
    }

    try {
      final String pid = entityMeta.votingProcesses?.active?.first;
      if (pid is String) {
        processMeta = await getProcessMetadata(pid, dvoteGw, web3Gw);
      }
    } on SocketException {
      error = Exception("Connecting to the Gateway failed");
    } on Exception catch (err) {
      if (err is String) {
        error = err;
        print(err);
      } else {
        error = Exception("Unable to retrieve the active processes");
      }
    }

    setState(() {
      _error = error;
      _loading = false;
      _entityMeta = entityMeta;
      _processMeta = processMeta;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (_loading) {
      return Scaffold(
        appBar: AppBar(
          title: const Text('Metadata'),
        ),
        body: Container(
          child: Center(child: Text("Loading...")),
        ),
      );
    } else if (_error != null) {
      return Scaffold(
        appBar: AppBar(
          title: const Text('Metadata'),
        ),
        body: Container(
          child: Center(child: Text(_error.toString())),
        ),
      );
    }

    return Scaffold(
      appBar: AppBar(
        title: const Text('Metadata'),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.all(16),
            child: _entityMeta == null
                ? null
                : Column(
                    children: <Widget>[
                      Text("ENTITY META:"),
                      SizedBox(height: 30),
                      Text(jsonEncode(_entityMeta.writeToJson())),
                    ],
                  ),
          ),
          _processMeta == null
              ? Divider()
              : Padding(
                  padding: EdgeInsets.all(16),
                  child: _entityMeta == null
                      ? null
                      : Column(
                          children: <Widget>[
                            Text("VOTING PROCESS"),
                            SizedBox(height: 30),
                            Text(jsonEncode(_processMeta?.writeToJson() ?? [])),
                          ],
                        ),
                ),
        ],
      ),
    );
  }
}
